% Alignment_main

% Author: Yongsoo Yang, UCLA Physics & Astronomy
%         yongsoo.ysyang@gmail.com

% This code contains an example of the tomographic tilt series alignment
% procedure, center of mass (CM) alignment method along the perpendicular
% to tilt axis direction, and common line alignment along the tilt axis
% direction.

% Depending on the nature of specific datasets, proper background
% subtraction should be also be performed along with any combination of
% these alignment methods.
%% load misaligned projections
Projs = importdata('Misaligned_Projs.mat'); % MxMxN array, M is the image size, N is # of projections
Angles = importdata('Angles.mat'); % corresponding tomography tilt angles

% For users who are interested in creating simulated projections from 3D
% objects, please see our tomographic reconstruction software package
% GENFIRE (website: genfire-em.com), where one can find MATLAB source codes
% for generating 2D projections from several example 3D volumes.

%% put the projection into a temporary array
temp_Projs = Projs;

%% CM alignment, perpendicular to tilt axis
% In this example, tilt axis is along the image horizontal (along the 2nd MATLAB 
% array dimention), so alignment is being done along image vertical.
% In case of common line not being applicable, CM method can also be applied
% along the tilt axis alignment.
% If the images are corrupted by strong background, CM method can easily
% fail, therefore proper background subtraction should be done first.

COMx_ref = round((size(Projs,1)+1)/2); % Reference CM point. To be used as input for
                                       % GENFIRE reconstruction, this
                                       % should be the value of "round((M+1)/2)".

for i=1:size(Projs,3)
    currImg = squeeze(temp_Projs(:,:,i)); % get current image
	[currCOMx,~] = My_COM(currImg); % calculate CM
    
    temp_Projs(:,:,i) = real(My_FourierShift(currImg, COMx_ref - currCOMx, 0)); % align to the reference CM point
end


%% Common line sub-pixel cross correlation alignment, along the tilt axis
% In this example, tilt axis is along the image horizontal (along the 2nd MATLAB 
% array dimention), so alignment is being done along image horizontal.

% index for reference projection for reference common line
RefInd = 10; % reference projection index
            % All other projection common lines will be aligned to the common
            % line of the reference projection.
Res = 0.1;  % sub-pixel search resolution (small value of Res may provide more 
            % accurate result in the cost of slower computation time, but
            % nose and other imperfecteness of data will eventually limit the
            % accuracy regardless of this value.

% re-order the for loop index based on the RefInd            
runInd = [RefInd 1:(RefInd-1) (RefInd+1):size(Projs,3)];
for i=runInd 
    currImg = squeeze(temp_Projs(:,:,i));axis image
    if i==RefInd % get reference common line
        refCMline = squeeze(sum(currImg,1));
    else % do cross correlation
        currCMline = squeeze(sum(currImg,1));
        [d1, maxVal] = My_Xcorr_subpixel_align_1D(refCMline, currCMline, Res); % sub-pixel cross correlation
        temp_Projs(:,:,i) = real(My_FourierShift(currImg, 0, -d1)); % align to the reference common line        
    end
end

%%
% the images in the variable "temp_Projs" is now aligned, ready for GENFIRE
% reconstruction.


%% Check alignment result
for i=1:length(Angles)
    subplot(1,2,1)
    imagesc(Projs(:,:,i));axis image;
    title(sprintf('misaligned projections, %6.2f deg',Angles(i)));
    subplot(1,2,2)
    imagesc(temp_Projs(:,:,i));axis image;
    title(sprintf('aligned projections, %6.2f deg',Angles(i)));
    pause(0.3)
end

%% Check common lines
clf
for i=1:length(Angles)
    subplot(1,2,1)
    plot(squeeze(sum(Projs(:,:,i),1)));hold on
    title('Common lines, misaligned');
    subplot(1,2,2)
    plot(squeeze(sum(temp_Projs(:,:,i),1)));hold on
    title('Common lines, aligned');
end
subplot(1,2,1)
hold off
subplot(1,2,2)
hold off