% My_Xcorr_subpixel_align_1D

% Author: Yongsoo Yang, UCLA Physics & Astronomy
%         yongsoo.ysyang@gmail.com

% This code runs the sub-pixel cross correlation to determine the spatial
% misalignment between two curves

% input paramters
% curve1: 1xn or nx1 array of real numbers
% curve2: 1xn or nx1 array of real numbers
% Res: Sub-pixel resolution, the sub-pixel iteration will stop until the
%      search stepsize is below this value. Should be less than 0.5.

% output paramters
% d1: the sub-pixel spatial misalignment value
% maxVal: the maximum cross-correlation value after alignment

function [d1, maxVal] = My_Xcorr_subpixel_align_1D(curve1, curve2, Res)

if Res >= 0.5
    disp('Resolution should be less than 0.5');
    return;
end

if size(curve1)~=size(curve2)
    disp('two curves should be in the same size!');
    return;
end

% initialize variables
curve1 = reshape(curve1, [1 length(curve1)]);
curve2 = reshape(curve2, [1 length(curve2)]);

Dim1 = size(curve1,2);
Dim1_pad = Dim1*2-1;

% zero pad curves for Fourier cross correlation
curve1_pad = My_paddzero(curve1,[1 Dim1_pad]);
curve2_pad = My_paddzero(curve2,[1 Dim1_pad]);

DivFact = 4;

NumIter = abs(floor(log(Res)/log(DivFact)));
curr_start1 = -0.5;
curr_end1 = 0.5;

N1_half = (Dim1_pad-1)/2;
KX = -N1_half:N1_half;

% Fourier transform
curve1_pad_FF = fftshift(fft(ifftshift(curve1_pad)));
curve2_pad_FF = fftshift(fft(ifftshift(curve2_pad)));

% sub-pixel iteration for maximum cross correlation
for i=1:NumIter
    
    % determine current iteration intervals
    curr_boundary1arr = curr_start1:(curr_end1-curr_start1)/DivFact:curr_end1;
    
    curr_ind1arr = (circshift(curr_boundary1arr,[0 -1]) + curr_boundary1arr)/2;
    
    curr_ind1arr = curr_ind1arr(1:end-1);

    % run cross correlation at each interval points
    maxXCar = zeros(1,length(curr_ind1arr));
    for j=1:length(curr_ind1arr)
        curr_Pfactor = exp(2*pi*1i*(curr_ind1arr(j)*KX/Dim1_pad));
        curr_Xcorr = abs(fftshift(ifft(ifftshift(conj(curve1_pad_FF).*curve2_pad_FF.*curr_Pfactor))));
        maxXCar(1,j) = max(curr_Xcorr(:));
    end
    
    % determine the maximim cross correlatoon
    [~, maxInd] = max(maxXCar(:));
    [~,maxj] = ind2sub(size(maxXCar),maxInd);
    
    % update next iteration start and end points for the intervals
    curr_start1 = curr_boundary1arr(maxj);
    curr_end1 = curr_boundary1arr(maxj+1);    
end

% sub-pixel shift & corresponding phase factor determined
sd1 = curr_ind1arr(maxj);
max_Pfactor = exp(2*pi*1i*(curr_ind1arr(maxj)*KX/Dim1_pad ));

% integer-pixel cross correlation
max_Xcorr = abs(fftshift(ifft2(ifftshift(conj(curve1_pad_FF).*curve2_pad_FF.*max_Pfactor))));
[maxVal, maxInd] = max(max_Xcorr(:));
[~, id1] = ind2sub(size(curve1_pad_FF),maxInd);
id1 = id1 - (N1_half+1);

% combine integer and sub-pixel shift result
d1 = id1+sd1;

end